// Copyright (C) 2003 Mooffie <mooffie@typo.co.il>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA.

#include <config.h>

#include "editbox.h"
#include "editor.h"
#include "dialogline.h"
#include "inputline.h"
#include "speller.h"

EditBox::action_entry EditBox::actions_table[] = {
    ADD_ACTION(EditBox, key_left,
	    N_("Move a character left")),
    ADD_ACTION(EditBox, key_right,
	    N_("Move a character right")),
    ADD_ACTION(EditBox, move_previous_line,
	    N_("Move to the previous line")),
    ADD_ACTION(EditBox, move_next_line,
	    N_("Move to the next line")),
    ADD_ACTION(EditBox, move_forward_page,
	    N_("Move to the next page")),
    ADD_ACTION(EditBox, move_backward_page,
	    N_("Move to the previous page")),
    ADD_ACTION(EditBox, move_forward_char,
	    N_("Move forward a character")),
    ADD_ACTION(EditBox, move_backward_char,
	    N_("Move back a character")),
    ADD_ACTION(EditBox, move_beginning_of_line,
	    N_("Move to the start of the current line")),
    ADD_ACTION(EditBox, move_end_of_line,
	    N_("Move to the end of the current line")),
    ADD_ACTION(EditBox, delete_backward_char,
	    N_("Delete the previous character")),
    ADD_ACTION(EditBox, delete_forward_char,
	    N_("Delete the character the cursor is on")),
    ADD_ACTION(EditBox, copy,
	    N_("Copy the selected text to clipboard")),
    ADD_ACTION(EditBox, cut,
	    N_("Copy the selected text to clipboard and delete it from the buffer")),
    ADD_ACTION(EditBox, paste,
	    N_("Paste the text in the clipboard into the buffer")),
    ADD_ACTION(EditBox, move_end_of_buffer,
	    N_("Move to the end of the buffer")),
    ADD_ACTION(EditBox, move_beginning_of_buffer,
	    N_("Move to the start of the buffer")),
    ADD_ACTION(EditBox, move_backward_word,
	    N_("Move to the start of the current or previous word")),
    ADD_ACTION(EditBox, move_forward_word,
	    N_("Move to the end of the current or next word")),
    ADD_ACTION(EditBox, delete_forward_word,
	    N_("Delete to the end of the current or next word")),
    ADD_ACTION(EditBox, toggle_dir_algo,
	    N_("Toggle the algorithm used to determine the base direction of paragraphs")),
    ADD_ACTION(EditBox, toggle_wrap,
	    N_("Toggle wrap type")),
    ADD_ACTION(EditBox, toggle_alt_kbd,
	    N_("Toogle Hebrew keyboard emulation")),
    ADD_ACTION(EditBox, set_translate_next_char,
	    N_("Translate next character")),
    ADD_ACTION(EditBox, justify,
	    N_("Justify the current or next paragraph")),
    ADD_ACTION(EditBox, delete_end_of_paragraph,
	    N_("Delete [to] end of paragraph")),
    ADD_ACTION(EditBox, undo,
	    N_("Undo the last change")),
    ADD_ACTION(EditBox, redo,
	    N_("Redo the last change you canceled")),
    ADD_ACTION(EditBox, delete_paragraph,
	    N_("Delete the current paragraph")),
    ADD_ACTION(EditBox, toggle_primary_mark,
	    N_("Start/cancel selection")),
    ADD_ACTION(EditBox, toggle_auto_justify,
	    N_("Enable/disable auto-justify")),
    ADD_ACTION(EditBox, toggle_auto_indent,
	    N_("Enable/disable auto-indent")),
    ADD_ACTION(EditBox, toggle_formatting_marks,
	    N_("Toggle display of formatting marks (paragraph ends, explicit BiDi marks, tabs)")),
    ADD_ACTION(EditBox, toggle_hebrew_points,
	    N_("Toggle display of Hebrew points (off/transliterated/as-is)")),
    ADD_ACTION(EditBox, toggle_maqaf,
	    N_("Toggle Hebrew maqaf highlighting and/or enable its ASCII transliteration")),
    ADD_ACTION(EditBox, toggle_smart_typing,
	    N_("Toggle smart-typing mode: auto replace some plain characters with typographical ones")),
    ADD_ACTION(EditBox, insert_maqaf,
	    N_("Insert Hebrew maqaf")),
    ADD_ACTION(EditBox, toggle_read_only,
	    N_("Toggle read-only status of buffer")),
    ADD_ACTION(EditBox, toggle_eops,
	    N_("Change end-of-paragraphs type")),
    END_ACTIONS
};

binding_entry EditBox::bindings_table[] = {
    { Event(KEY_LEFT), "key_left" },
    { Event(KEY_RIGHT), "key_right" },
    { Event(KEY_UP), "move_previous_line" },
    { Event(KEY_DOWN), "move_next_line" },
    { Event(KEY_NPAGE), "move_forward_page" },
    { Event(KEY_PPAGE), "move_backward_page" },
    { Event(KEY_HOME), "move_beginning_of_line" },
    { Event(KEY_END), "move_end_of_line" },
    { Event(CTRL, 'b'), "move_backward_char" },
    { Event(CTRL, 'f'), "move_forward_char" },
    { Event(CTRL, 'p'), "move_previous_line" },
    { Event(CTRL, 'n'), "move_next_line" },
    { Event(CTRL, 'a'), "move_beginning_of_line" },
    { Event(CTRL, 'e'), "move_end_of_line" },
    { Event(KEY_BACKSPACE), "delete_backward_char" },
    { Event(KEY_DC), "delete_forward_char" },
    { Event(KEY_F(12)), "toggle_alt_kbd" },
    { Event(ALT, '>'), "move_end_of_buffer" },
    { Event(ALT, '<'), "move_beginning_of_buffer" },
    { Event(ALT, 'b'), "move_backward_word" },
    { Event(ALT, 'f'), "move_forward_word" },
    { Event(ALT, 'F'), "toggle_formatting_marks" },
    { Event(ALT, 'n'), "toggle_hebrew_points" },
    { Event(ALT, 'd'), "delete_forward_word" },
    { Event(ALT, 't'), "toggle_dir_algo" },
    { Event(ALT, 'w'), "toggle_wrap" },
    { Event(ALT, 'h'), "toggle_alt_kbd" },
    { Event(CTRL, 'q'), "set_translate_next_char" },
    { Event(CTRL, 'j'), "justify" },
    { Event(CTRL, 'k'), "delete_end_of_paragraph" },
    { Event(CTRL, 'r'), "redo" },
    { Event(CTRL, 'u'), "undo" },
    { Event(CTRL, 'c'), "copy" },
    { Event(CTRL, 'x'), "cut" },
    { Event(CTRL, 'v'), "paste" },
    { Event(CTRL, 'y'), "delete_paragraph" },
    { Event(CTRL, '^'), "toggle_primary_mark" },
    { Event(CTRL, '@'), "toggle_primary_mark" },
    { Event(KEY_F(11)), "toggle_primary_mark" }, // cygwin
    { Event(ALT, 'J'), "toggle_auto_justify" },
    { Event(ALT, 'i'), "toggle_auto_indent" },
    { Event(ALT, 'k'), "toggle_maqaf" },
    { Event(ALT, 'q'), "toggle_smart_typing" },
    { Event(ALT, '-'), "insert_maqaf" },
    { Event(ALT, 'R'), "toggle_read_only" },
    { Event(CTRL | ALT, 'e'), "toggle_eops" },
    END_BINDINGS
};

Editor::action_entry Editor::actions_table[] = {
    ADD_ACTION(Editor, layout_windows, NULL),
    ADD_ACTION(Editor, load_file,
	    N_("Load file from disk")),
    ADD_ACTION(Editor, save_file,
	    N_("Save file to disk")),
    ADD_ACTION(Editor, save_file_as,
	    N_("Save file in another name")),
    ADD_ACTION(Editor, insert_file,
	    N_("Insert file from disk into the current buffer")),
    ADD_ACTION(Editor, change_tab_width,
	    N_("Change the TAB character size")),
    ADD_ACTION(Editor, change_justification_column,
	    N_("Change the column used to justify paragraphs")),
    ADD_ACTION(Editor, insert_unicode_char,
	    N_("Insert a specific Unicode character using its known code number")),
    ADD_ACTION(Editor, go_to_line,
	    N_("Go to a specific line")),
    ADD_ACTION(Editor, search_forward,
	    N_("Search for a string, starting from the cursor")),
    ADD_ACTION(Editor, search_forward_next,
	    N_("Search for the next occurrence of the string")),
    ADD_ACTION(Editor, toggle_cursor_position_report,
	    N_("Enable/disable continuous display of cursor position in the status line")),
    ADD_ACTION(Editor, refresh_and_center,
	    N_("Repaint the terminal screen (if it was garbled for some reason)")),
    ADD_ACTION(Editor, show_character_code,
	    N_("Print the unicode value and UTF-8 sequence of the character the cursor is on")),
    ADD_ACTION(Editor, show_character_info,
	    N_("Print the corresponding UnicodeData.txt line of the character the cursor is on")),
    ADD_ACTION(Editor, quit,
	    N_("Quit the editor")),
    ADD_ACTION(Editor, help,
	    N_("Get help for using this editor")),
    ADD_ACTION(Editor, describe_key,
	    N_("Gives a short description for a shortcut key")),
    ADD_ACTION(Editor, change_directory,
	    N_("Change current directory")),
    ADD_ACTION(Editor, toggle_arabic_shaping,
	    N_("Toggle Arabic shaping")),
    ADD_ACTION(Editor, write_selection_to_file,
	    N_("Write selection to file")),
    ADD_ACTION(Editor, change_scroll_step,
	    N_("Change the scroll step")),
    ADD_ACTION(Editor, load_unload_speller,
	    N_("Explicitly load or unload a speller process")),
    ADD_ACTION(Editor, spell_check_all,
	    N_("Spell check all the document")),
    ADD_ACTION(Editor, spell_check_forward,
	    N_("Spell check the document, from the cursor onward")),
    ADD_ACTION(Editor, spell_check_word,
	    N_("Spell check the word on which the cursor is")),
    END_ACTIONS
};

binding_entry Editor::bindings_table[] = {
#ifdef KEY_RESIZE
    { Event(KEY_RESIZE), "layout_windows" },
#endif
    { Event(KEY_F(1)), "help" },
    { Event(KEY_F(2)), "save_file" },
    { Event(CTRL, 's'), "save_file_as" },
    { Event(CTRL, 'w'), "write_selection_to_file" },
    { Event(KEY_F(3)), "load_file" },
    { Event(KEY_F(7)), "search_forward" },
    { Event(KEY_F(17)), "search_forward_next" },
    { Event(ALT, 'r'), "insert_file" },
    { Event(CTRL | ALT, 'c'), "change_directory" },
    { Event(ALT, 'x'), "quit" },
    { Event(ALT, 'X'), "quit" },
    { Event(ALT, 'c'), "toggle_cursor_position_report" },
    { Event(CTRL | ALT, 't'), "change_tab_width" },
    { Event(CTRL | ALT, 'j'), "change_justification_column" },
    { Event(CTRL | ALT, 'v'), "insert_unicode_char"},
    { Event(CTRL | ALT, 'u'), "show_character_code"},
    { Event(CTRL | ALT, 's'), "change_scroll_step"},
    { Event(ALT, 'a'), "toggle_arabic_shaping"},
    { Event(ALT, '\t'), "show_character_info"},
    { Event(ALT, 'l'), "go_to_line"},
    { Event(CTRL, 'l'), "refresh_and_center"},
    { Event(KEY_F(4)), "describe_key"},
    { Event(KEY_F(5)), "spell_check_all" },
    { Event(KEY_F(6)), "spell_check_forward" },
    { Event(ALT, '$'), "spell_check_word" },
    { Event(ALT, 'S'), "load_unload_speller" },
    END_BINDINGS
};

DialogLine::action_entry DialogLine::actions_table[] = {
    ADD_ACTION(DialogLine, layout_windows, NULL),
    ADD_ACTION(DialogLine, cancel_modal, NULL),
    ADD_ACTION(DialogLine, refresh, NULL),
    END_ACTIONS
};

binding_entry DialogLine::bindings_table[] = {
#ifdef KEY_RESIZE
    { Event(KEY_RESIZE), "layout_windows" },
#endif
    { Event(CTRL, 'g'), "cancel_modal" },
    { Event(CTRL, 'c'), "cancel_modal" },
    { Event(CTRL, 'l'), "refresh" },
    END_BINDINGS
};

InputLine::action_entry InputLine::actions_table[] = {
    ADD_ACTION(InputLine, previous_completion, NULL),
    ADD_ACTION(InputLine, next_completion, NULL),
    ADD_ACTION(InputLine, end_modal, NULL),
    ADD_ACTION(InputLine, previous_history, NULL),
    ADD_ACTION(InputLine, next_history, NULL),
    END_ACTIONS
};

binding_entry InputLine::bindings_table[] = {
    { Event(0, '\r'), "end_modal" },
    { Event(0, '\t'), "next_completion" },
    { Event(ALT, '\t'), "previous_completion" },
    { Event(CTRL, 'p'), "previous_history" },
    { Event(CTRL, 'n'), "next_history" },
    { Event(KEY_UP), "previous_history" },
    { Event(KEY_DOWN), "next_history" },
    END_BINDINGS
};

HelpBox::action_entry HelpBox::actions_table[] = {
    ADD_ACTION(HelpBox, end_modal, NULL),
    ADD_ACTION(HelpBox, layout_windows, NULL),
    ADD_ACTION(HelpBox, refresh_and_center, NULL),
    END_ACTIONS
};

binding_entry HelpBox::bindings_table[] = {
#ifdef KEY_RESIZE
    { Event(KEY_RESIZE), "layout_windows" },
#endif
    { Event(CTRL, 'l'), "refresh_and_center"},
    { Event(KEY_F(1)), "end_modal" },
    { Event(ALT, 'x'), "end_modal" },
    { Event(ALT, 'X'), "end_modal" },
    END_BINDINGS
};

SpellerWnd::action_entry SpellerWnd::actions_table[] = {
    ADD_ACTION(SpellerWnd, layout_windows, NULL),
    ADD_ACTION(SpellerWnd, ignore_word, NULL),
    ADD_ACTION(SpellerWnd, abort_spelling, NULL),
    ADD_ACTION(SpellerWnd, abort_spelling_restore_cursor, NULL),
    ADD_ACTION(SpellerWnd, add_to_dict, NULL),
    ADD_ACTION(SpellerWnd, edit_replacement, NULL),
    ADD_ACTION(SpellerWnd, set_global_decision, NULL),
    ADD_ACTION(SpellerWnd, refresh, NULL),
    END_ACTIONS
};

binding_entry SpellerWnd::bindings_table[] = {
#ifdef KEY_RESIZE
    { Event(KEY_RESIZE), "layout_windows" },
#endif
    { Event(CTRL, 'g'),	"abort_spelling" },
    { Event(CTRL, 'c'),	"abort_spelling" },
    { Event(ALT, 'x'),	"abort_spelling_restore_cursor" },
    { Event(0, 'q'),	"abort_spelling_restore_cursor" },
    { Event(0, 'x'),	"abort_spelling_restore_cursor" },
    { Event(CTRL, 'l'), "refresh" },
    { Event(0, ' '), "ignore_word" },
    { Event(0, 'a'), "add_to_dict" },
    { Event(0, 'r'), "edit_replacement" },
    { Event(0, 'g'), "set_global_decision" },
    END_BINDINGS
};

