// Copyright (C) 2003 Mooffie <mooffie@typo.co.il>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA.

#include <config.h>

#include "event.h"
#include "my_wctob.h"
#include "dbg.h"

void Event::print()
{
    DBG(2, ("EVENT: "));
    if (is_literal())
	DBG(2, ("literal: '%lc'\n", ch));
    else {
	DBG(2, ("comp: "));
	if (modifiers) {
	    if (modifiers & CTRL)
		DBG(2, ("CTRL-"));
	    if (modifiers & ALT)
		DBG(2, ("ALT-"));
	    if (modifiers & SHIFT)
		DBG(2, ("SHIFT-"));
	}
	if (ch)
	    DBG(2, ("'%lc'\n", ch));
	else
	    DBG(2, ("%s\n", keyname(keycode)));
    }
}

#ifdef HAVE_WIDE_CURSES
static void low_level_get_wch(Event &evt, WINDOW *wnd)
{
    wint_t c;
    int ret;
    do {
	ret = wget_wch(wnd, &c);
	if (ret == KEY_CODE_YES) {
	    evt.ch = 0;
	    evt.keycode = (int)c;
	} else if (ret == OK) {
	    evt.keycode = 0;
	    evt.ch = (unichar)c;
	}
    } while (ret == ERR);
}
#else
static void low_level_get_wch(Event &evt, WINDOW *wnd)
{
    int ret;
    do {
	ret = wgetch(wnd);
	if (ret >= 256) {
	    evt.ch = 0;
	    evt.keycode = ret;
	} else {
	    evt.keycode = 0;
	    evt.ch = terminal::force_iso88598 ? iso88598_to_unicode(ret) : BTOWC(ret);
	}
    } while (ret == ERR);
}
#endif

static void get_base_event(Event &evt, WINDOW *wnd)
{
    evt.type = evtKbd;
    evt.modifiers = 0;
    low_level_get_wch(evt, wnd);
    if (evt.keycode == 0 && evt.ch < 32) {
	switch (evt.ch) {
	case 9:
	case 13:
	case 27:
	    // leave alone TAB, ENTER, ESC
	    break;
	case 0:
	case 28:
	case 29:
	case 30:
	case 31:
	    evt.ch += 'A' - 1;
	    evt.modifiers = CTRL;
	    break;
	default:
	    evt.ch += 'a' - 1;
	    evt.modifiers = CTRL;
	}
    }
}

void get_next_event(Event &evt, WINDOW *wnd)
{
    get_base_event(evt, wnd);
    if (evt.ch == 27) {
	// emulate ALT
	get_base_event(evt, wnd);
	evt.modifiers |= ALT;
    }
}

